@@ -1,6 +1,8 @@
 # -*- coding:utf-8 -*-
-
+# Auther: Zed
 """
+It's beautiful right? ^_^
+
 dm = Matrix()
 print('data_matrix:', dm)
 res = dm[dm['code'] == '000001']
@@ -30,10 +32,50 @@ data_matrix_slice:
 300  0.008385  0.087556  0.994873  0.706558  0.752703
 400  0.388965  0.946414  0.967339  0.567995  0.071485
 
+update:
+    2022-12-02 23:52  :
+        Increased function matrix.T(axis_one,axis_two), then you can transform two axis,just like pd.DataFrame().T
+        dm.T(0, 2)
+        print('-' * 30)
+        print('transform!')
+        print(dm)
 
+        dm.T('code', 'timestamp')
+        print('-' * 30)
+        print('transform!')
+        print(dm)
+    2022-12-03 0:31  :
+        Increased params type in function matrix.T, then you can use 'list' as params
+        dm.T(['code','timestamp','features'])
+        print('-' * 30)
+        print('transform!')
+        print(dm)
+    2022-12-04 22:49  :
+        Increased handle df_dict to np.ndarray and axis_name, then user can import df list easily to matrix
+        e.g:
+            data_list = os.listdir()
+            data_list = [data.split('.')[0] for data in data_list if data.endswith('.csv')]
+
+            df_dict = {}
+            for data in data_list:
+                df = pd.read_csv(data + '.csv')
+                df.index = df["trade_date"]  # 请确保dataframe.index是数据的时间,或者是有意义且不重复的值
+                df_dict[data] = df
+            dm = Matrix(df_dict, axis_name=['code', 'datetime', 'features'])
+    next plan:
+            1. Update dm element: dm.append(matrix0/DataFrame) or matrix2=em.concat([matrix0,matrix1])
+            2. IO stream: dm.read/to pandas_df,pkl,csv
+            3. Increase logic operation such as '>= <= > < !=/~ and & or / in' : dm[20220105<=datetime and code in ['000001','000002']]
+            4. Increase more __magic_function__ such as __del__: del dm["feature=='close'"] / dm.delete('datetime<=20220105')
+            5. Query function: dm.query("20220105<=datetime<=20220506 and code in ['000001','000002'] and feature.capital>float(1e9)")
+            6. Increase function fill_na: dm.fill_na('forward_value/backward_value/zero_value/avg_value')
+            7. Del dm[dm[::3]:ClassMatrix]
+            -1. More simular pd.DataFrame operation function in Matrix
 """
+
 import numpy as np
 from typing import Union, NewType
+from copy import deepcopy
 import pandas as pd
 
 set_print_limit = False  # 显示有限的输出设置
@@ -42,6 +84,92 @@ _SeriesType = Union[dict, tuple, list]
 _SeriesTypeBase = [tuple, list]
 
 
+def row_fill_na(data: pd.DataFrame, fill_list) -> pd.DataFrame:
+    _na_data = np.full([len(fill_list), data.shape[1]], np.nan)
+
+    new_data = np.vstack((data, _na_data))
+
+    _index = list(data.index)
+    _index.extend(fill_list)
+    new_df = pd.DataFrame(new_data, index=_index, columns=data.columns)
+    new_df.sort_index(inplace=True)
+    # new_df = pd.concat([data, new_df])
+    return new_df
+
+
+def columns_fill_na(data: pd.DataFrame, fill_list) -> pd.DataFrame:
+    _na_data = np.full([data.shape[0], len(fill_list)], np.nan)
+    new_data = np.hstack((data, _na_data))
+    _columns = list(data.columns)
+    _columns.extend(fill_list)
+    new_df = pd.DataFrame(new_data, index=data.index, columns=_columns)
+
+    return new_df
+
+
+def diff_calcu(total_list, _array_list):
+    _diff_list = list(set(total_list) - set(_array_list))
+    return sorted(_diff_list)
+
+
+def df_dict2matrix(df_dict, axis_name=None):
+    if axis_name is None:
+        axis_name = ['code', 'datetime', 'features']
+    index = []
+    row = []
+    columns = []
+
+    df_info_dict = {}
+    for key, value in df_dict.items():
+        index.append(key)
+
+        row.extend(list(value.index.values))
+        columns.extend(list(value.columns.values))
+        df_info_dict[key] = {
+            'row': list(value.index.values),
+            'columns': list(value.columns.values),
+        }
+    # def value_fill_na_check():
+    values_dict = {}
+    for key, value in deepcopy(df_info_dict).items():
+        diff_row = diff_calcu(row, value['row'])
+        diff_columns = diff_calcu(columns, value['columns'])
+
+        if diff_row:
+
+            new_df = row_fill_na(df_dict[key], diff_row)
+
+            if diff_columns:
+                new_df = columns_fill_na(new_df, diff_columns)
+            new_df.sort_index(inplace=True)
+            values_dict[key] = new_df
+            continue
+        if diff_columns:
+            new_df = columns_fill_na(df_dict[key], diff_columns)
+            new_df.sort_index(inplace=True)
+            values_dict[key] = new_df
+        else:
+            df_dict[key].sort_index(inplace=True)
+            values_dict[key] = df_dict[key]
+
+    first_data_key = list(values_dict.keys())[0]
+    first_data = values_dict[first_data_key]
+
+    index = list(values_dict.keys())
+    row = list(first_data.index)  # 时间记得排序
+    columns = list(first_data.columns)
+
+    axis_name = {axis_name[0]: index, axis_name[1]: row, axis_name[2]: columns}
+    _array = np.empty([1, len(row), len(columns)])
+    # print(_array)
+
+    for key, value in values_dict.items():
+        # print(value)
+        _array = np.vstack((_array, [value.values]))
+    _array = _array[1:]
+    return _array, axis_name
+
+
 def check_Series_unit(series):
     _len = 0
     _type = None
@@ -91,7 +219,7 @@ def check_Series_dtype(dtype):
     return dtype
 
 
-
+# axis_name = ['timestamp', 'code', 'features']
 def _print(data: _SeriesType, head=0, tail=0):
     _print_len = 10
     if not head:
@@ -222,13 +350,14 @@ class Series:
         return _data + info
 
     def __repr__(self):
-        return self
+        return str(self)
 
 
 
 
 _MatrixGetSingleItemType = Union[str, int, Series]
 _MatrixGetMultiItemType = Union[str, int, Series]
+_MatrixAxisNameType = Union[dict, list]
 MatrixGetDataType = Union[np.ndarray, list, tuple, dict, None]
 
 
@@ -265,19 +394,31 @@ class DataFrame:
 
 
 class Matrix:
-
-    def __init__(self, data: MatrixGetDataType = None, axis_name: dict = None, unique=True):
+    # 可接收多个pd.DataFrame聚合为Matrix数据类型
+    # 标准样例：第一维度为时间，第二维度为合约标的代码，第三维度为特征因子features
+    # 均需要外部提供，标准协议为：
+    # axis_name={'timestamp':[xx,xx,xx],
+    #            'code':[xx,xx,xx],
+    #            'features':[xx,xx,xx],
+    #           }
+    # 需要解决一个问题：如果两个dataframe的长度不一致时如何解决
+    def __init__(self, data: MatrixGetDataType = None, axis_name: _MatrixAxisNameType = None, unique=True):
         # unique如果能确保值是唯一的，那么可提高性能
-        self._dimension_dict = {}
+
+        self._dimension_dict = {}  # 轴名为key，维度为value
+        self._dimension_num_dict = {}  # 维度为key，轴名为value
         self.unique = unique
 
         if type(data) == np.ndarray:
             data: np.ndarray
             if len(data.shape) < 3:
                 raise ValueError(f'error data dimension:{len(data.shape)}')
-        # ToDo elif type(data) == np.ndarray:
+        elif type(data) == dict:
+            data, axis_name = df_dict2matrix(data, axis_name=axis_name)
 
-        if not axis_name or not data:
+        # list, tuple, dict 内部装着pd.dataframe
+
+        elif type(data) != np.ndarray or not axis_name:
             # test data:
             axis_name = {'timestamp': [100, 200, 300, 400], 'code': ['000001', '000002', '000003'],
                          'features': ['close', 'high', 'low', 'open', 'ret']}
@@ -290,16 +431,97 @@ class Matrix:
         self.data = data
         self.axis_name = axis_name
 
-        # ToDo 未来self.index 将变为Series类型
         self.index = None
         self.index_name = None
         self.row = None
         self.row_name = None
         self.columns = None
         self.columns_name = None
+
+        self._axis_name_dict = {}
+        self._axis_name_key = []
+        self.init()
+
+    # 转置的概念：要借助numpy进行0，1，2轴任意转置
+    def pandas_df_to_matrix(self, df_dict: dict):
+        # 把多个pandas.dataframe数据放入df_dict中，然后循环处理为numpy.ndarray矩阵，最终变为matrix类
+
+        pass
+
+    def T(self, axis_one, axis_two=None):
+        if type(axis_one) == int and type(axis_two) == int:
+            axis_one_num, axis_two_num = axis_one, axis_two
+
+        elif type(axis_one) == str and type(axis_two) == str:
+            axis_one_num, axis_two_num = self._dimension_dict[axis_one], self._dimension_dict[axis_two]
+
+        elif type(axis_one) == list and not axis_two:
+            if len(axis_one) == 3:
+                if axis_one[0] == axis_one[1] or axis_one[1] == axis_one[2] or axis_one[0] == axis_one[2]:
+                    raise ValueError(f'Error value, element should not be same value {axis_one}')
+                if type(axis_one[0]) == str and type(axis_one[1]) == str and type(axis_one[2]) == str:
+                    _one, _two, _three = self._dimension_dict[axis_one[0]], self._dimension_dict[axis_one[1]], \
+                                         self._dimension_dict[axis_one[2]]
+                    axis_one = [_one, _two, _three]
+                elif type(axis_one[0]) == int and type(axis_one[1]) == int and type(axis_one[2]) == int:
+                    _d = [0, 1, 2]
+                    if axis_one[0] not in _d or axis_one[1] not in _d or axis_one[2] not in _d:
+                        raise ValueError(f'Error value element in {axis_one}, it should be between in 0-2')
+
+                else:
+                    raise ValueError(f'Error type element in {axis_one}, it should be str or in')
+                new_queue = axis_one
+                self.data = self.data.transpose(new_queue)
+                self._T(new_queue)
+                return self
+            else:
+                raise ValueError(f"Error param length: {axis_one}, it's length should be 3")
+        else:
+            raise ValueError(
+                f'Error type param: {axis_one, axis_two}, it should double int or str or axis_one is list and axis_two is None ')
+
+        self.data = self.data.swapaxes(axis_one_num, axis_two_num)
+
+        if axis_one_num == 0 and axis_two_num == 1 or axis_one_num == 1 and axis_two_num == 0:
+            new_queue = [1, 0, 2]
+
+        elif axis_one_num == 0 and axis_two_num == 2 or axis_one_num == 2 and axis_two_num == 0:
+            new_queue = [2, 1, 0]
+        elif axis_one_num == 1 and axis_two_num == 2 or axis_one_num == 2 and axis_two_num == 1:
+            new_queue = [0, 2, 1]
+        else:
+            raise ValueError(f'error axis_num or axis_name: {axis_one, axis_two}')
+        self._T(new_queue)
+        return self
+
+    def _T(self, new_queue):
+        _dimension_dict = {}
+        _axis_name_dict = {}
+        for num in new_queue:
+            _axis_name = self._dimension_num_dict[num]
+            _axis_name_dict[_axis_name] = self.axis_name[_axis_name]
+        self.axis_name = _axis_name_dict
+        self.init()
+
+
+    def append(self,auto_fix=False):
+        # auto_fix=True:如果用户能确保加入的维度和Matrix.value维度一样，那么就不需要作维度相等判断，因此性能会更高
+        pass
+
+    def concat(self):
+        # 拼接两个matrix数据
+        pass
+
+    def init(self):
+
+        if len(self.data.shape) != 3:
+            raise ValueError(f'error data shape {self.data}')
+        if len(self.axis_name.keys()) != 3:
+            raise ValueError(f'error axis_name len {self.data}')
         i = 0
-        for key, value in axis_name.items():
+        for key, value in self.axis_name.items():
             self._dimension_dict[key] = i
+            self._dimension_num_dict[i] = key
             if i == 0:
                 self.index = value
                 self.index_name = key
@@ -310,35 +532,9 @@ class Matrix:
                 self.columns = value
                 self.columns_name = key
             i += 1
-        self._axis_name_dict = {}
-        self._axis_name_key = []
-        self.init()
+        self._axis_name_to_range()
 
-    def pandas_df_to_matrix(self, df_dict: dict):
-        # 把多个pandas.dataframe数据放入df_dict中，然后循环处理为numpy.ndarray矩阵，最终变为matrix类
-
-        pass
-
-    def update(self):
-
-        pass
-
-    def append(self):
-
-        pass
-
-    def concat(self):
-
-        pass
-
-    def init(self):
-        if len(self.data.shape) != 3:
-            raise ValueError(f'error data shape {self.data}')
-        if len(self.axis_name.keys()) != 3:
-            raise ValueError(f'error axis_name len {self.data}')
-        self.axis_name_to_range()
-
-    def axis_name_to_range(self):
+    def _axis_name_to_range(self):
         _data_dict = {}
         for key, value in self.axis_name.items():
             # 用于实现类似loc精确定位矩阵元素
@@ -365,13 +561,14 @@ class Matrix:
 
             item: _MatrixGetSingleItemType
             if type(item) == str:
-
+                # dm['data_xx']:返回某维度所有index
                 if item not in self._axis_name_key:
                     raise ValueError(f'{item} not exist in Matrix')
 
                 return Series(list(self._axis_name_dict[item].keys()), item, self.unique)
             elif type(item) == Series:
 
+                # Series类用于提供定位信息，根据dtype又分为bool和loc
                 if item.dtype == 'bool':
 
                     if item.name not in self._dimension_dict:
@@ -472,7 +669,6 @@ class Matrix:
 
                             return DataFrame(_res[0], df_axis_name)
 
-
             elif type(item) == list:
 
                 if self.unique:
@@ -481,7 +677,7 @@ class Matrix:
         else:
 
             print('_MatrixGetMultiItemType')
-
+            pass
 
     @property
     def values(self):
@@ -493,7 +689,7 @@ class Matrix:
         _reverse_dimension = {v: k for k, v in self._dimension_dict.items()}
         columns_str = list(self._axis_name_dict[_reverse_dimension[2]].keys())
         if len(columns_str) > 10:
-            columns_str = str(columns_str[:5])[:-1] + '...' + str(columns_str[:5])[1:]
+            columns_str = str(columns_str[:2])[:-1] + '...' + str(columns_str[:2])[1:]
         _print_data = "\n             " + str(columns_str)
 
         for i, value in enumerate(self.data):
@@ -515,7 +711,7 @@ class Matrix:
                     _print_data += '                  ......\n'
                     break
 
-                _print_data += list(self._axis_name_dict[_reverse_dimension[1]].keys())[j]
+                _print_data += str(list(self._axis_name_dict[_reverse_dimension[1]].keys())[j])
                 _value = value[j]
                 # 控制列
                 if len(_value) >= 5:
@@ -530,33 +726,4 @@ class Matrix:
         return self._print()
 
 
-def axis_name_to_range(data_dict):
-    _data_dict = {}
-    for key, value in data_dict.items():
-        _data_dict[key] = dict(zip(value, list(range(len(value)))))
 
-    return _data_dict
-
-
-if __name__ == '__main__':
-    # Matrix Demo————————————————————————————
-    print('-' * 30)
-    print('res')
-    import time
-    t1 = time.time()
-    mat = Matrix()
-    # 从三维数据中查找轴名为code，元素为000001的矩阵切片
-    axis_name = 'code'  # code timestamp
-    compare_obj = '000001'  # '000001' 200
-    print(mat)
-    print(mat[axis_name])
-    print(mat[axis_name] == compare_obj)
-    print(mat[mat[axis_name] == compare_obj])
-    print('data_matrix:', mat)
-    res = mat[mat[axis_name] == compare_obj]
-    df = mat[mat[axis_name] == compare_obj].to_pandas_df()  # 切片数据转为pd.DataFrame
-    print('data_matrix_slice:\n', res)
-    print('to_pandas_df:\n', df)
-
-    t2 = time.time()
-    print('slice obj cost time:', t2 - t1)
